/**********************************************************************
Copyright (c) 2002 IBM Corporation and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the Common Public License v1.0
which accompanies this distribution, and is available at
http://www.eclipse.org/legal/cpl-v10.html
Contributors:
Adrian Colyer, Andy Clement, Tracy Gardner - initial version
...
**********************************************************************/
package org.eclipse.ajdt.internal.core;

import java.io.File;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.jdt.ui.JavaElementImageDescriptor;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Point;
import org.eclipse.ajdt.ui.AspectJPlugin;

/**
 * A utility class to capture all those little functions that
 * keep cropping up. Converting IPaths to fully-qualified native
 * filenames being one of the most common!!
 */
public class AJDTUtils {

	// icon sizes
	private static final Point SMALL_SIZE= new Point(16, 16);
	private static final Point BIG_SIZE= new Point(22, 16);
	private static final int SMALL_ICONS_MASK = 0x020;


	/**
	 * Return the fully-qualifed native OS path of the workspace.
	 * e.g. D:\eclipse\workspace
	 */
	public static String getWorkspacePath( ) {
		return AspectJPlugin.getWorkspace().getRoot().getLocation().toOSString();
	}
	
	/**
	 * Return the fully-qualified name of the root directory for a
	 * project.
	 */
	public static String getProjectRootDirectory( IProject project ) {
		return project.getLocation().toOSString();	
	}
	
	/**
	 * Return the fully-qualified native OS path of a project resource
	 */
	public static String getResourcePath( IResource resource ) {
		return resource.getLocation( ).toOSString();
	}

	/**
	 * decorate an icon given a set of adornment flags.
	 * Constants for flag values are defined in JavaElementImageDescriptor
	 */
	public static ImageDescriptor decorate( ImageDescriptor base, int decorations ) {
		Point size= useSmallSize(decorations) ? SMALL_SIZE : BIG_SIZE;
		return new JavaElementImageDescriptor( base, decorations, size);	
	}

	/**
	 * determine which icon size to use for a given set of decorations
	 */
	private static boolean useSmallSize(int flags) {
		return (flags & SMALL_ICONS_MASK) != 0;
	}	
}
