
package org.eclipse.ajdt.internal.ui.wizards;

import java.awt.SystemColor;
import java.net.URL;
import java.net.MalformedURLException;
import org.eclipse.swt.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.jface.action.*;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ajdt.ui.AspectJPlugin;
import org.eclipse.ajdt.internal.core.AJDTUtils;
import org.eclipse.ajdt.internal.core.resources.AJDTIcon;
import org.eclipse.ajdt.internal.ui.*;
import org.eclipse.ajdt.ui.*;
import org.eclipse.ajdt.internal.ui.*;
import org.eclipse.ui.*;
import org.aspectj.asm.*;
import org.aspectj.compiler.base.ast.ASTObject;
import org.aspectj.ajde.*;
import org.eclipse.jdt.internal.ui.*;
import org.eclipse.jdt.internal.ui.javaeditor.*;
import java.text.MessageFormat;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Hashtable;
import java.io.File;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.ajdt.internal.core.resources.*;
import org.eclipse.ajdt.internal.ui.editor.*;
import org.aspectj.asm.*;

public class JoinPointProbeView extends ViewPart {
	
	public static JoinPointProbeView INSTANCE = null;
	Composite _parent;
	private String workspaceDir = null;
	private String currentFile;
	private Display currentDisplay;
	Tree tree;
	Hashtable treeItemLocationMap = new Hashtable( );
	
	public JoinPointProbeView() {
		super();
		
		try {
			JoinPointProbeView.INSTANCE = this;
			currentDisplay = Display.getCurrent( );
				
			IPath workspaceDirPath = ResourcesPlugin.getWorkspace( ).getRoot( ).getLocation( );
			String prefix = workspaceDirPath.toString( ).replace( '/','\\');
			workspaceDir = prefix.toUpperCase().substring(0,1) + prefix.substring(1);

			INSTANCE = this;
		} catch (Exception e) {
			e.printStackTrace();	
		}
	}

	public void updateView() {
		createTree(_parent);
	}	

	public Composite getParent() {
		return _parent;	
	}

	void createTree(Composite parent) {
		try {
			treeItemLocationMap = new Hashtable( );
			if (tree != null) { 
				tree.removeAll(); 
			} else {
				tree = new Tree(parent, SWT.MULTI);
			}
			tree.addListener( SWT.Selection, new TreeSelectionListener( ) );
			
			boolean probeDefined = true;
			
			StructureNode rootNode = null;
			if (probeDefined) {
				rootNode = Ajde.getDefault().getStructureViewManager().createViewForSourceFile(
					JoinPointProbeWizard.getProbeFilePath(), 
					Ajde.getDefault().getStructureViewManager().getDefaultViewProperties()
				).getRootNode().getStructureNode();
				
				StructureNode aspectNode = (StructureNode)rootNode.getChildren().get(0);
				ProgramElementNode adviceNode = null;
				List relations = new ArrayList();
				List children = new ArrayList();
				String label = "<no probe pointcut>";
				
				if (aspectNode.getChildren().size() > 0) {
					adviceNode = (ProgramElementNode)aspectNode.getChildren().get(0);
					relations = adviceNode.getRelations();
	        		children = adviceNode.getChildren();
	        		label = "<probe pointcut>";
				} 
	        	ProgramElementNode probeNode = new ProgramElementNode(
	        		label, 
	        		ProgramElementNode.Kind.POINTCUT, 
	        		new ArrayList(), 
	        		ProgramElementNode.Accessibility.PUBLIC, 
	        		"", 
	        		"", 
	        		"", 
	        		null,
	        		relations,
	        		children,
	        		false, 
	        		null);
	        
				if (rootNode == null) {
				 	createTreeNode(tree, StructureModel.NO_STRUCTURE);
				} else {
					createTreeNode( tree, probeNode );	
				}
			} else {
				createTreeNode(tree, StructureModel.NO_STRUCTURE);
			}
		} catch (Exception e) {
			e.printStackTrace();	
		}
	}
  

	/**
	 * Callback for IUIActivityListener
	 */
	public void activeFileChanged( String fileName ) {
		if ( fileName != null ) {
			currentFile = fileName;
			System.out.println( "OutlineView - active file is now: " + currentFile );	
			// TO DO: find out how to get prefix programmatically
			createTree( _parent );
		} else {
			System.out.println( "OutlineView - got a null active file call" );
		}
	}	

	// ------------------------------------------------------------------------
	// methods driven from ViewPart parent class

	/**
	 * @see WorkbenchPart#setFocus()
	 */
	public void setFocus() {
		// to do:
		// give the focus to the tree view inside the outline
	}

	/**
	 * @see WorkbenchPart#createPartControl(Composite)
	 */	
	public void createPartControl(Composite parent) {
		_parent = parent;
		createTree(parent);
//		registerToolbarActions();
	}

	// ------------------------------------------------------------------------
	// access to JDT resource strings - private in JDT so replicated here
	// - yeurgh!

	private static final String RESOURCE_BUNDLE =
		"org.eclipse.jdt.internal.ui.javaeditor.JavaEditorMessages";
	//$NON-NLS-1$

	private static ResourceBundle fgResourceBundle =
		ResourceBundle.getBundle(RESOURCE_BUNDLE);

	private static String getJDTString(String key) {
		try {
			return fgResourceBundle.getString(key);
		} catch (MissingResourceException e) {
			return "!" + key + "!"; //$NON-NLS-2$ //$NON-NLS-1$
		}
	}
	

	/**
	 * get the image descriptor for a toolbar icon
	 */
	private ImageDescriptor getImageDescriptor(String relativePath) {
		String iconPath = "icons/";
		try {
			AspectJPlugin plugin = AspectJPlugin.getDefault();// .getPluginInstance();
			URL installURL = plugin.getDescriptor().getInstallURL();
			URL url = new URL(installURL, iconPath + relativePath);
			return ImageDescriptor.createFromURL(url);
		} catch (MalformedURLException e) {
			return ImageDescriptor.getMissingImageDescriptor();
		}
	}

	/**
	 * called from actions to toggle sorting
	 */
	private void setSorting(boolean sortingOn) {
		// TO DO
	}
	
	/**
	 * inner class to manage tree selections
	 */
	class TreeSelectionListener implements org.eclipse.swt.widgets.Listener {
	
		public void handleEvent( Event event ) {
			TreeItem tItem = (TreeItem) event.item;
			SourceLocation loc = (SourceLocation) treeItemLocationMap.get( tItem );
			if ( loc != null ) {
				try {
					File sourceFile = new File(loc.getSourceFile() );				
					IResource ir = AspectJPlugin.getDefault().getAjdtProjectProperties().findResource( sourceFile.getAbsolutePath() );
					System.out.println( "IResource = " + ir );
					IMarker im = ir.createMarker( IMarker.LINE_NUMBER );
					im.setAttribute( IMarker.LINE_NUMBER, loc.getBeginLine( ) );
					AspectJPlugin.getDefault().getActiveWorkbenchWindow().getActivePage( ).openEditor( im );				
				} catch ( org.eclipse.core.runtime.CoreException cEx )  {
					System.err.println( cEx );	
				}						
			} 
		}
		
	};
	
	
	class SourceLocation {
		
		private String sourceFile;
		private int beginLine;
		
		public String getSourceFile( ) { return sourceFile; }
		public int getBeginLine( ) { return beginLine; }
		
		public SourceLocation( String file, int line ) {
			sourceFile = file;
			beginLine = line;
		}
		
	};


	/**
	 * Utility method to see what I'm getting from the structure manager
	 * From observation, kind can be one of:
	 * project, package, file, class, field, method, aspect, pointcut, advice,
	 * constructor
	 * (and possibly introduction, declaration - not contained in my test file)
	 */
	private void dumpNode(StructureNode node, int indent) {
		for (int i = 0; i < indent; i++) {
			System.err.print(" ");
		}
		System.err.println(node.getKind() + " : " + node.getName());
		List children = node.getChildren();
		if (children != null) {
			for (Iterator it = children.iterator(); it.hasNext();) {
				StructureNode child = (StructureNode) it.next();
				dumpNode(child, indent + 2);
			}
		}
	}

	/**
	 * Utility method to dump text version of stucture node into a tree
	 */
	void createTreeNode( Widget parent, StructureNode node ) {		
		TreeItem tItem;
		// this is a hack to cover the Tree / TreeItem issue
		if ( parent instanceof Tree ) {
			tItem = new TreeItem( ((Tree) parent), SWT.NULL );
		} else {
			tItem = new TreeItem( ((TreeItem) parent), SWT.NULL );
		}
		String nodeText = node.getName( );
		Image nodeImage = getImageForNode( node );
		tItem.setText( nodeText );
		tItem.setImage( nodeImage );
		if ( node instanceof ProgramElementNode ) {
			ProgramElementNode pNode = (ProgramElementNode) node;
			createRelations( tItem, pNode );
			if (node.getSourceLocation() != null) {
				treeItemLocationMap.put( 
					tItem, 
					new SourceLocation( pNode.getSourceLocation().getSourceFilePath(), pNode.getSourceLocation().getLineNumber()) );
			}
		}		
		List children = node.getChildren();
		if (children != null) {
			for (Iterator it = children.iterator(); it.hasNext();) {
				StructureNode child = (StructureNode) it.next();
				createTreeNode( tItem, child );
			}
		}				
	    tItem.setExpanded( true );
	}


	/**
	 * return the image to use for the given StructureNode
	 */
	private Image getImageForNode( StructureNode node ) {
		String kind = node.getKind( );
		if (node instanceof ProgramElementNode) {
			Image image = createNodeImage((ProgramElementNode)node);
			return image;	
		} else {
			return null;		
		}
	}
	
	/**
	 * add the relationships into the tree for a program element
	 */
	private void createRelations( TreeItem tItem, ProgramElementNode pNode ) {
		List relations = pNode.getRelations();
		if (relations != null) {
			for (Iterator it = relations.iterator(); it.hasNext();) {
				RelationNode relation = (RelationNode) it.next();
				String relId = relation.getRelation().toString( );
				TreeItem relItem = new TreeItem( tItem, SWT.NULL );
				String relName = relation.getName( );
				if ( relName.equals( "imports") ) { relName = "import declarations"; }
				relItem.setText( relName );				
				relItem.setImage( createNodeImage(relation.getRelation()));
				createRelationChildren( relItem, relation );
			}
		}			
	}

	private void createRelationChildren( TreeItem item, RelationNode rNode ) {
		List children = rNode.getChildren();
		if (children != null ) {
			for (Iterator cit = children.iterator( ); cit.hasNext( ); ) {
				LinkNode lNode = (LinkNode) cit.next( );
				ProgramElementNode pNode = lNode.getProgramElementNode();
				
				TreeItem tItem = new TreeItem( item, SWT.NULL );
				tItem.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_BLUE));
				tItem.setText( lNode.toString( ) );
				tItem.setImage( getImageForNode( pNode ) );
				treeItemLocationMap.put( tItem, 
						            new SourceLocation( pNode.getSourceLocation().getSourceFilePath(), pNode.getSourceLocation().getLineNumber() ) );
			}
		}		
	}


	private Image createNodeImage(Relation relation) {
		ImageDescriptor baseDescriptor;
		AJDTIcon icon = (AJDTIcon) AspectJImages.registry().getRelationIcon(
			relation
		);		
		baseDescriptor = icon.getImageDescriptor();	
		return AJDTUtils.decorate( baseDescriptor, 0).createImage();
	}

	private Image createNodeImage(ProgramElementNode node) {
		ImageDescriptor baseDescriptor;
		AJDTIcon icon = (AJDTIcon) AspectJImages.registry().getStructureIcon(
			node.getProgramElementKind(),
			ProgramElementNode.Accessibility.PUBLIC	
		);		
		baseDescriptor = icon.getImageDescriptor();	
		return AJDTUtils.decorate( baseDescriptor, 0).createImage();
	}
	

	/**
	 * Called whenever AJDE Structure model changes
	 */
	private void refreshTree() {
		// TO DO
	}

//	/**
//	 * create tree (called from createPartControl)
//	 */
//	void createTree(Composite parent);// {
//		createImageMap();
//		treeItemLocationMap = new Hashtable( );
//		if (tree != null) { 
//			tree.removeAll(); 
//		} else {
//			tree = new Tree(parent, SWT.MULTI);
//		}
//		tree.addListener( SWT.Selection, new TreeSelectionListener( ) );
//		
//		//loadStructureModel( );
//		// (structure mode now loaded from constructor and on model update)
//		
//		StructureNode rootNode = null;
//		String fileToDisplay = workspaceDir + currentFile;
//		System.out.println( "OutlineView displaying tree for: " + fileToDisplay );
//		rootNode = Ajde.getDefault().getStructureModelManager().getStructureModel( fileToDisplay );
//		
//		
//		// Even this doesn't work - getLatActiveFile returns null, or if I play
//		// with the editor, gives me a file path in which there is no eay way to 
//		// determine where directories stop and package names begin.
// 		
// 		//String currFile = UIActivityListener.getLastActiveFile();
//
//		//dumpNode(rootNode, 0); 
//		createTreeNode( tree, rootNode );		
//	}

//	void createImageMap( ) {
//		if ( imageMap == null ) {
//			imageMap = new Hashtable( );
//			imageMap.put( "project", getImageDescriptor("project.gif").createImage() );	
//			imageMap.put( "package", JavaPluginImages.get( JavaPluginImages.IMG_OBJS_PACKAGE) );	
//			imageMap.put( "file", JavaPluginImages.get( JavaPluginImages.IMG_OBJS_CUNIT) );	
//			imageMap.put( "class", JavaPluginImages.get( JavaPluginImages.IMG_OBJS_CLASS) );	
//			imageMap.put( "field.private",  JavaPluginImages.get( JavaPluginImages.IMG_MISC_PRIVATE ) );
//			imageMap.put( "field.protected", JavaPluginImages.get( JavaPluginImages.IMG_MISC_PROTECTED ) );
//			imageMap.put( "field.public",  JavaPluginImages.get( JavaPluginImages.IMG_MISC_PUBLIC ) );
//			imageMap.put( "field.package", JavaPluginImages.get( JavaPluginImages.IMG_MISC_DEFAULT ) );
//			imageMap.put( "method.private", JavaPluginImages.get( JavaPluginImages.IMG_MISC_PRIVATE) );	
//			imageMap.put( "method.protected", JavaPluginImages.get( JavaPluginImages.IMG_MISC_PROTECTED) );	
//			imageMap.put( "method.public", JavaPluginImages.get( JavaPluginImages.IMG_MISC_PUBLIC) );	
//			imageMap.put( "method.package", JavaPluginImages.get( JavaPluginImages.IMG_MISC_DEFAULT) );	
//			imageMap.put( "aspect", getImageDescriptor("aspect.gif").createImage() );	
//			imageMap.put( "pointcut", org.eclipse.ajdt.internal.core.resources.AspectJImages.E_ADVICE.getImageDescriptor().getImageData());//getImageDescriptor("joinPoint.gif").createImage() );	
//			imageMap.put( "advice", getImageDescriptor("advice.gif").createImage() );	
//			imageMap.put( "constructor.private",  JavaPluginImages.get( JavaPluginImages.IMG_MISC_PRIVATE ) );
//			imageMap.put( "constructor.protected", JavaPluginImages.get( JavaPluginImages.IMG_MISC_PROTECTED ) );
//			imageMap.put( "constructor.public",  JavaPluginImages.get( JavaPluginImages.IMG_MISC_PUBLIC ) );
//			imageMap.put( "constructor.package", JavaPluginImages.get( JavaPluginImages.IMG_MISC_DEFAULT ) );
//			imageMap.put( "import declarations", JavaPluginImages.get( JavaPluginImages.IMG_OBJS_IMPCONT) );
//			imageMap.put( "pointcut used by", getImageDescriptor( "referenceBack.gif").createImage( ) );
//			imageMap.put( "uses pointcut", getImageDescriptor( "referenceForward.gif").createImage( ));
//			imageMap.put( "affects constructors", getImageDescriptor( "adviceForward.gif").createImage( ) );
//			imageMap.put( "affects methods",getImageDescriptor( "adviceForward.gif").createImage( ) );
//			imageMap.put( "inherits", getImageDescriptor( "inheritanceForward.gif").createImage( ));
//			imageMap.put( "inherits members",getImageDescriptor( "inheritanceForward.gif").createImage( ) );
//			imageMap.put( "is inherited by",getImageDescriptor( "referenceBack.gif").createImage( ) );
//			imageMap.put( "constructor affected by",getImageDescriptor( "adviceBack.gif").createImage( ) );
//			imageMap.put( "method affected by",getImageDescriptor( "adviceBack.gif").createImage( ) );
//			// introductions and declarations				
//		}		
//	}
}
  