package org.eclipse.ajdt.internal.ui.wizards;


import org.aspectj.asm.RelationNode;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.ui.*;
import org.eclipse.ui.help.*;
import org.eclipse.ui.internal.*;
import org.eclipse.ui.internal.model.AdaptableList;
import org.eclipse.ui.internal.registry.*;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.*;
import org.eclipse.jface.wizard.ProgressMonitorPart;
import org.eclipse.jface.dialogs.*;
import org.eclipse.swt.widgets.Composite;
import java.io.*;
import org.aspectj.ajde.*;
import org.eclipse.ajdt.internal.builder.*;
import org.eclipse.ajdt.ui.*;
import org.eclipse.core.runtime.*;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.jdt.core.JavaCore;
 
/**
 * The export wizard allows the user to choose which nested export wizard to run.
 * The set of available wizards comes from the export wizard extension point.
 */
public class JoinPointProbeWizard extends Wizard {
	
	//public static String PROTOTYPE_PROBE = "C:/Apps/eclipse/runtime-workspace/Figures/_AnonymousJoinpointProbe.java";
	
	public static final String PROBE_SUFFIX = "_JoinPointProbe.java";
	
	private static final String NAME = "Joinpoint Probe Wizard";
	
	private JoinPointProbeWizardPage wizardPage = new JoinPointProbeWizardPage();
	private IWorkbench workbench;

	public static String getProbeFilePath() {
		return Ajde.getDefault().getProjectProperties().getProjectSourcePath()
			+ File.separator + Ajde.getDefault().getProjectProperties().getProjectName() 
			+ PROBE_SUFFIX;
	}
		

	public JoinPointProbeWizard() {
		addPage(wizardPage);
	}

	protected AdaptableList getAvailableExportWizards() {
		return new WizardsRegistryReader(IWorkbenchConstants.PL_EXPORT).getWizards();
	}

	public void init(IWorkbench aWorkbench) {
		this.workbench = aWorkbench;
		//this.selection = currentSelection;

		setWindowTitle(NAME);
		setDefaultPageImageDescriptor(
			WorkbenchImages.getImageDescriptor(
				IWorkbenchGraphicConstants.IMG_WIZBAN_NEW_WIZ));
		setNeedsProgressMonitor(true);
	}

	public boolean performFinish() {
		if (JoinpointProbeBuilder.probeIsValid) {
			showProbeView();
		} else {
			MessageDialog.openInformation(workbench.getActiveWorkbenchWindow().getShell(), 
				"Invalid Probe", "Probe was invalid, run the wizard again to try another.");
		}	
       	
       return true;
	}
	
	
	private void showProbeView() {
		try {
			JoinPointProbeView.INSTANCE.updateView();
//			IViewPart[] views = workbench.getActiveWorkbenchWindow().getActivePage().getViews();
//			IViewPart[] views = workbench.getActiveWorkbenchWindow().getActivePage().getViewReferences();
			workbench.getActiveWorkbenchWindow().getActivePage().showView(
				"org.eclipse.ajdt.internal.ui.wizards.JoinPointProbeView"
			);
		} catch (PartInitException pie) {
            Ajde.getDefault().getErrorHandler().handleError("Could not generate join point probe.", pie);
        }
	}
}

