
package org.eclipse.ajdt.internal.ui.wizards;

import java.io.*;

import java.util.*;

import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.events.*;
import org.eclipse.ajdt.internal.builder.*;
import org.eclipse.ajdt.ui.AspectJPlugin;
import org.eclipse.jdt.internal.ui.util.SWTUtil;
import org.aspectj.ajde.*;

class JoinPointProbeWizardPage extends WizardPage {

	private JoinpointProbeBuilder builder = new JoinpointProbeBuilder();
	
	private Button		evaluateButton;
	
	private Composite	parent;
	private Composite	pageContentsGroup;	
	private Label		nameLabel;
	private Text		nameText;
	private Label		pcdLabel;
	private Text		pcdText;
	private Label	    directionsLabel;

	private final static String PAGE_NAME= "Joinpoint Probe Wizard";
	private final static String PAGE_DESCRIPTION = "Creates a joinpoint probe that can be used to find crosscutting modularity.";
	
	public JoinPointProbeWizardPage() {
		super(PAGE_NAME);
		setTitle(PAGE_NAME); 
		setDescription(PAGE_DESCRIPTION); 
		
		//builder.addListener(new ProbeBuildListener(this));

		//setPageComplete(false);
	}

	public void createControl(Composite parent) {
		this.parent = parent;
		Composite composite= new Composite(parent, SWT.NULL);
		composite.setLayout(new GridLayout());
		composite.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_FILL | GridData.HORIZONTAL_ALIGN_FILL));

		createPageContents(composite);
		setControl(composite);
	}

	public Composite getParent() {
		return parent;	
	}

	protected void createPageContents(Composite parent) {
		
		initializeDialogUnits(parent);
		
		Composite pageContentsGroup = new Composite(parent, SWT.NONE);
		GridLayout layout= new GridLayout();
		pageContentsGroup.setLayout(layout);

		layout.numColumns = 1;
		
		pageContentsGroup.setLayout(layout);
		pageContentsGroup.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
		directionsLabel = new Label(pageContentsGroup, SWT.NONE);
		directionsLabel.setText("The pointcut designator that you entere will be " 
			+ "verified for correctness when you click \"Next\"."); 

		Composite filedsGroup = new Composite(parent, SWT.NONE);
		GridLayout fieldsLayout = new GridLayout();
		//layout.marginHeight= 0;
		fieldsLayout.numColumns = 2;
		filedsGroup.setLayout(fieldsLayout);
		filedsGroup.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		nameLabel= new Label(filedsGroup, SWT.NONE);
		nameLabel.setText("Joinpoint probe name: "); 
		nameLabel.setEnabled(false);

		nameText= new Text(filedsGroup, SWT.SINGLE | SWT.BORDER);
		GridData nameData = new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL);
		nameData.widthHint= convertWidthInCharsToPixels(40);
		nameText.setLayoutData(nameData);
		nameText.setEnabled(false);

		pcdLabel = new Label(filedsGroup, SWT.TOP);
		pcdLabel.setText("Pointcut designator for probe: "); 

		pcdText = new Text(filedsGroup, SWT.MULTI | SWT.BORDER);
		GridData pcdData = new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL);
		pcdData.widthHint = convertWidthInCharsToPixels(40);
		pcdData.heightHint = convertWidthInCharsToPixels(20);
		pcdText.setLayoutData(pcdData);		


		Composite actionsGroup = new Composite(parent, SWT.NONE);
		GridLayout actionsLayout = new GridLayout();
		actionsLayout.numColumns = 1;
		actionsGroup.setLayout(actionsLayout);
		actionsGroup.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

		evaluateButton= new Button(actionsGroup, SWT.PUSH);
		evaluateButton.setText("     Evaluate Probe     "); 
		evaluateButton.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		SWTUtil.setButtonDimensionHint(evaluateButton);
		evaluateButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleEvaluateButtonPressed();
			}
		});
	}

//	public String getPcdText() {
//		return pcdText.getText();			
//	}

	private void handleEvaluateButtonPressed() {
		writeProbe(pcdText.getText());
		buildProbe();	
	}

	private void writeProbe(String probePcd) {
		String probe = 
			"aspect _AnonymousJoinpointProbe { \n"
			+ "    before(): " + probePcd + " \n{ }\n"
			+ "}";

        try {
            FileOutputStream fos = new FileOutputStream(JoinPointProbeWizard.getProbeFilePath(), false);
            fos.write(probe.getBytes());
            fos.close();
        } catch (IOException ioe) {
            Ajde.getDefault().getErrorHandler().handleError("Could not write joinpoint probe.", ioe);
        }	
	}
	
	private boolean buildProbe() {

		//AspectJPlugin.getDisplay().asyncExec( new Runnable( ) {
		//	public void run( ) {
		String configFile = Ajde.getDefault().getProjectProperties().getProjectSourcePath()
			+ File.separator + ".." + File.separator 
			+ Ajde.getDefault().getProjectProperties().getProjectName()  
			+ "_Probe.lst";
		
		String contents = JoinPointProbeWizard.getProbeFilePath() + "\n";
		java.util.List files = AspectJPlugin.getDefault().getAjdtProjectProperties().getProjectSourceFiles(AspectJPlugin.getDefault().getCurrentProject());
		for (Iterator it = files.iterator(); it.hasNext(); ) {
			contents += it.next() + "\n";
		}
			
		
		try {
			FileWriter writer = new FileWriter(configFile, false);
			writer.write(contents + "\n");
			writer.close();
		} catch (IOException ioe) {
			
		}
		
		builder.build(configFile);
		
//		MessageDialog.openInformation(workbench.getActiveWorkbenchWindow().getShell(), 
//			"Join Point Probe Status", "Proceed to build the probe (no progress monitor will show).");
//		try {
//			ProgressMonitorPart pm = new ProgressMonitorPart(JoinPointProbeView.INSTANCE.getParent(), new GridLayout());
//			pm.setVisible(true);
//			AspectJPlugin.getCurrentProject().build(
//			IncrementalProjectBuilder.FULL_BUILD, AspectJPlugin.BUILDER_ID, null, pm);
//		} catch (CoreException ce) {
//            Ajde.INSTANCE.getErrorHandler().handleError("Could not build join point probe.", ce);
//        }
		return true;
	}


//	private void handleEvaluateButtonPressed() {
//		String pcd = pcdText.getText();
//		System.err.println("> evaluating: " + pcd);
//		writeProbe(pcd);
//		boolean success = true;
//	
//		if (success) {
//			super.setPageComplete(true);
//		}
//	}

	public final void saveWidgetValues() {

	}

	/**
	 *	Hook method for restoring widget values to the values that they held
	 *	last time this wizard was used to completion.
	 */
	protected void restoreWidgetValues() {

	}	


	/**
	 * Creates a new label with a bold font.
	 *
	 * @param parent the parent control
	 * @param text the label text
	 * @return the new label control
	 */
	protected Label createLabel(Composite parent, String text, boolean bold) {
		Label label= new Label(parent, SWT.NONE);
		if (bold)
			label.setFont(JFaceResources.getBannerFont());
		label.setText(text);
		GridData data= new GridData();
		data.verticalAlignment= GridData.FILL;
		data.horizontalAlignment= GridData.FILL;
		label.setLayoutData(data);
		return label;
	}

	/**
	 * Creates a horizontal spacer line that fills the width of its container.
	 *
	 * @param parent the parent control
	 */
	protected void createSpacer(Composite parent) {
		Label spacer= new Label(parent, SWT.NONE);
		GridData data= new GridData();
		data.horizontalAlignment= GridData.FILL;
		data.verticalAlignment= GridData.BEGINNING;
		spacer.setLayoutData(data);
	}
}