package org.eclipse.ajdt.internal.ui.wizards;

import java.io.*;
import java.util.*;
import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.jface.viewers.*;
import org.eclipse.jface.window.*;
import org.eclipse.jface.action.*;
import org.eclipse.core.runtime.*;
import org.eclipse.ajdt.internal.ui.wizards.JoinPointProbeWizard;
import org.eclipse.core.resources.*;
import org.eclipse.jdt.core.*;
import org.eclipse.ui.*;
import org.eclipse.jface.wizard.WizardDialog;

import org.aspectj.ajde.*;
 

public class JoinpointProbeAction
	implements ISelectionChangedListener, IWorkbenchWindowActionDelegate {

	private IWorkbenchWindow activeWindow = null;

	// When a user clicks the icon in the toolbar or kicks of AJBrowser from the
	// popup menu, the current project is ascertained from the AspectjPlugin and
	// then 'recurse' is called to move through all resources in that project - 
	// detecting which are .java files.  Any found during this recursion are
	// added to the java_files vector.  When the top level recurse() call returns
	// we know the vector contains all the java files in the project and its
	// contains make a suitable .lst file for passing to AJBrowser
	private Vector java_files = null;

	public void recurse(IResource[] resource_list) {
		try {
			for (int i = 0; i < resource_list.length; i++) {
				IResource ir = resource_list[i];
				if (ir.getName().endsWith(".java")) {
					System.err.println(
						"Java Source file found: Path=" + ir.getLocation().toOSString());
					java_files.addElement(new File(ir.getLocation().toOSString()));
				}
				if (ir instanceof IContainer) {
					recurse(((IContainer) ir).members());
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	// Executed when button clicked or popup menu "Start AJBrowser" clicked
	public void run(IAction action) {
		try {
			JoinPointProbeWizard jpWizard = new JoinPointProbeWizard();
			jpWizard.init(activeWindow.getWorkbench());
			//jpWizard.
			WizardDialog dialog = new WizardDialog(activeWindow.getShell(), jpWizard);
			dialog.open();
		} catch (Throwable t) {
			t.printStackTrace();
			Ajde.getDefault().getErrorHandler().handleError("Could not start wizard", t);
		}

	}

	public void activateLauncher(Window window, IStructuredSelection selection) {
		if (window instanceof IWorkbenchWindow) {
			((IWorkbenchWindow) window).getActivePage().saveAllEditors(true);
		}
	}

	public void selectionChanged(SelectionChangedEvent selection) {
	}

	/* Receives events when selections change in the views, if a IJavaElement
	 * is selected, we make its project the current project (ready for launching
	 * ajbrowser) - notice if a multiple selection is made, we only pay attention
	 * to the project that the first selected item belongs to.
	 */

	public void selectionChanged(IAction action, ISelection selection) {
		boolean state = !selection.isEmpty();

		//    if (selection instanceof IStructuredSelection) {
		//      IStructuredSelection structuredSelection = (IStructuredSelection) selection;
		//      Object first = structuredSelection.getFirstElement();
		//      if (first instanceof IJavaElement) {
		//      	IJavaProject ijp = ((IJavaElement)first).getJavaProject();
		//      	AspectjPlugin.setCurrentProject(ijp.getElementName());
		//      	System.err.println("Current proj="+ijp.getElementName());
		//      }
		//    }
	}

	public void dispose() {
	}

	// Initial work on looking at which editors are currently open, so we can
	// move to the current lines when 'clicked' on in the AJBrowser.

	public void refreshEditorList() {
		if (activeWindow == null) {
			System.err.println("No window currently marked active");
			return;
		}
		IWorkbenchPage iwp = activeWindow.getActivePage();
		IEditorPart[] ieps = iwp.getEditors();
		if (ieps != null) {
			for (int i = 0; i < ieps.length; i++) {
				System.err.println("ieps[" + i + "] is " + ieps[i].getTitle());
			}
		}
	}

	public void init(IWorkbenchWindow window) {
		activeWindow = window;
	}
}